# NOMAD Domain-specific Examples

Build upon your understanding of NOMAD's features with domain-specific examples and explanations.

<div markdown="block" class="home-grid">
<div markdown="block">

### Computational data

Historically a repository for Density Functional Theory calculations, NOMAD now supports a wide range of computational methodologies including advanced many-body calculations and classical molecular dynamics simulations, as well as complex simulation workflows.

- [Quick Start: Uploading computational data](./computational_data/uploading.md)
- [Standard and custom computational workflows](./computational_data/workflows.md)
- [Guide to computational MetaInfo](./computational_data/metainfo.md)
- [Guide to computational schema plugins](./computational_data/schema_plugins.md)
- [Guide to computational parser plugins](./computational_data/parser_plugins.md)
- H5MD: Uploading custom molecular dynamics data
    - [How-to](./computational_data/h5md_howto.md)
    - [Explanation](./computational_data/h5md_expl.md)
    - [Reference](./computational_data/h5md_ref.md)
<!--
#### subcategory 1
- links...
#### subcategory 2
- links... -->

</div>
<div markdown="block">

### Experimental data

Thanks to key activities of the FAIRmat project, NOMAD also supports a set of parsing capabilities for standardizing data from materials characterization experiments.

- [NeXus](./experiment_data/nexus.md)
- [Guide to the pynxtools parser library](./experiment_data/pynxtools.md)
- [Electron microscopy](./experiment_data/em.md)
- [Photoemission spectroscopy](./experiment_data/mpes.md)
- [X-ray photoemission spectroscopy](./experiment_data/xps.md)
- [Optical spectroscopy](./experiment_data/opt.md)
- [Atom probe tomography](./experiment_data/apm.md)
- [Scanning tunneling spectroscopy](./experiment_data/sts.md)

</div>

<div markdown="block">

</div>
</div>

<h2>One last thing</h2>

If you can't find what you're looking for in our guides, [contact our friendly team](mailto:support@nomad-lab.eu) for personalized help and assistance. Don't worry, we're here to help and learn what we're doing wrong!
